==> doar-0.0.6.tar.gz: doar-0.0.6/src/doar/key_stream.h <==
#ifndef DOAR_KEY_STREAM_H
#define DOAR_KEY_STREAM_H

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <vector>

namespace Doar {
  // MEMO: keyは'\0'終端文字列で、値が0xFFの文字を含むことはできない
  class KeyStream {
  public:
    KeyStream(){}
    KeyStream(const char* key) : cur(key) {}
    
    Code read() { return static_cast<unsigned char>(*cur++); }
    const char* rest() const { return cur; }
    bool eos() const { return cur[-1]=='\0'; }

  private:
    const char* cur;
  };

  class KeyStreamList {
  public:
    KeyStreamList(const char* filepath) 
      : buf(NULL),valid(false) {
      int f = open(filepath, O_RDONLY);
      if(f==-1)
	return;
      valid=true;
      
      struct stat statbuf;
      fstat(f, &statbuf);
      
      buf = new char[statbuf.st_size+1];
      ::read(f, buf, statbuf.st_size);
      close(f);
      
      buf[statbuf.st_size]='\0';
      
      init(buf,buf+statbuf.st_size-1);
    }
    KeyStreamList(const char** strs, unsigned str_count) 
      : buf(NULL),valid(true) {
      words.resize(str_count);
      for(unsigned i=0; i < str_count; i++)
	words[i] = KeyStream(strs[i]);
    }
    ~KeyStreamList() { delete [] buf; }

    std::size_t size() const { return words.size(); }
    KeyStream& operator[](std::size_t idx) { return words[idx]; }

    operator bool() const { return valid; }
    
  private:
    void init(char* cur, const char* end) {
      words.push_back(cur);
      for(cur=strchr(cur,'\n'); cur; cur=strchr(cur,'\n')) {
	*cur = '\0';
	cur++;
	if(cur < end)
	  words.push_back(KeyStream(cur));
      }
    }
    
  private:
    char* buf;
    std::vector<KeyStream> words;
    
    bool valid;
  };
}

#endif

==> doar-0.0.13.tar.gz: doar-0.0.13/src/doar/key_stream.h <==
#ifndef DOAR_KEY_STREAM_H
#define DOAR_KEY_STREAM_H

#include "types.h"
#include <vector>
#include <cstdio>

namespace Doar {
  class KeyStream {
  public:
    // NOTE: key is NULL terminated string.  And it can't contain '0xFF' character.
    KeyStream(const char* key) : cur(key) {}
    KeyStream(){}
    
    Code read() { return static_cast<unsigned char>(*cur++); }
    const char* rest() const { return cur; }
    bool eos() const { return cur[-1]=='\0'; }
    
  private:
    const char* cur;
  };

  class KeyStreamList {
  public:
    // construct from file
    KeyStreamList(const char* filepath) 
      : buf(NULL),valid(false) {
      FILE* f;
      if((f=fopen(filepath,"rb"))==NULL)
	return;
    
      fseek(f,0,SEEK_END);
      long file_size = ftell(f); // NOTE: File size limit is 2^(sizeof(long)-1) byte
      fseek(f,0,SEEK_SET);
      
      if(file_size != -1){
	buf = new char[file_size+1];
	fread(buf, sizeof(char), file_size, f);
	buf[file_size]='\0';
	init(buf,buf+file_size-1);
	valid=true;
      }
    
      fclose(f);
    }
    // construct from string array
    KeyStreamList(const char** strs, uint32 str_count) 
      : words(strs,strs+str_count), buf(NULL), valid(true) {}
    ~KeyStreamList() { delete [] buf; }

    std::size_t size() const { return words.size(); }
    KeyStream& operator[](std::size_t idx) { return words[idx]; }

    operator bool() const { return valid; }
    
  private:
    void init(char* cur, const char* end) {
      words.push_back(cur);
      for(cur=strchr(cur,'\n'); cur; cur=strchr(cur,'\n')) {
	*cur = '\0';
	cur++;
	if(cur < end)
	  words.push_back(KeyStream(cur));
      }
    }
    
  private:
    std::vector<KeyStream> words;
    char* buf;
    bool valid;
  };
}

#endif
